@@ -1,6 +1,7 @@
 {-# LANGUAGE ScopedTypeVariables #-}
 module Verify.Graphics.Vty.DisplayRegion ( module Verify.Graphics.Vty.DisplayRegion
                                          , module Graphics.Vty.DisplayRegion
+                                         , DebugWindow(..)
                                          )
     where
 
@@ -21,7 +22,7 @@ instance Show EmptyWindow where
 
 instance Arbitrary DebugWindow where
     arbitrary = do
-        w <- suchThat arbitrary (/= 0)
-        h <- suchThat arbitrary (/= 0)
+        w <- choose (1,1024)
+        h <- choose (1,1024)
         return $ DebugWindow w h
 
